/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * PALETTE.C -- Useful library for VGA Palette manipulation.               *
 *                                                                         *
 * by Gary Neal, Jr. -- garyneal@geocities.com                             *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "include\palette.h"

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * SetPalette                                                              *
 *                                                                         *
 * Sets a single palette index to a specified color attribute.             *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void SetPalette(int attribute, union ColorAttribute color)
{
    /* Disable interrupts */
    disable();                            /* Same as: asm cli; */

    /* Send color palette register index to write to */
    outp(PALETTE_WRITE, attribute);

    /* Write the color palette entry */
    outp(PALETTE_DATA, color.RGB.red);    /* Red   intensity */
    outp(PALETTE_DATA, color.RGB.green);  /* Green intensity */
    outp(PALETTE_DATA, color.RGB.blue);   /* Blue  intensity */

    /* Enable interrupts */
    enable();                             /* Same as: asm sti; */
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * SetPaletteUsing                                                         *
 *                                                                         *
 * Sets the entire VGA palette to specified color attributes.              *
 * Array must contain enough entries to set the entire palette.            *
 *                                                                         *
 * If NULL was passed, function call is ignored.                           *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void SetPaletteUsing(union ColorAttribute *color)
{
    int i;

    /* If no array was passed, leave */
    if (color == NULL) return;

    /* Disable interrupts */
    disable();                            /* Same as: asm cli; */

    /* Send color palette register index to write to */
    outp(PALETTE_WRITE, 0);

    /* Here comes the palette */
    for (i = 0; i < ScreenColors; i++) {

        /* Write the color palette entry */
        outp(PALETTE_DATA, color[i].RGB.red);    /* Red   intensity */
        outp(PALETTE_DATA, color[i].RGB.green);  /* Green intensity */
        outp(PALETTE_DATA, color[i].RGB.blue);   /* Blue  intensity */

    }

    /* Enable interrupts */
    enable();                             /* Same as: asm sti; */
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * GetPaletteUsing                                                         *
 *                                                                         *
 * Gets a single color attribute from the VGA palette index.               *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
union ColorAttribute GetPalette(int attribute)
{
    union ColorAttribute color;

    /* Disable interrupts */
    disable();                            /* Same as: asm cli; */

    /* Write color palette register index to read from */
    outp(PALETTE_READ, attribute);

    /* Read color palette entry */
    color.color     = 0L;
    color.RGB.red   = inp(PALETTE_DATA);
    color.RGB.green = inp(PALETTE_DATA);
    color.RGB.blue  = inp(PALETTE_DATA);

    /* Enable interrupts */
    enable();                             /* Same as: asm sti; */

    return color;                         /* Return the color attribute */
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * GetPaletteUsing                                                         *
 *                                                                         *
 * Gets the entire VGA palette information into specified color array.     *
 * Array must contain enough entries to get the entire palette.            *
 *                                                                         *
 * If NULL was passed, GetPaletteUsing tries to create an array            *
 * (using malloc), your program must later free the allocated array.       *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
union ColorAttribute *GetPaletteUsing(union ColorAttribute *color)
{
    int i;

    /* If no array was passed, create one */
    if (color == NULL) {
        color = (union ColorAttribute *)
                malloc(ScreenColors * sizeof(union ColorAttribute));
        if (color == NULL) return NULL;
    }

    /* Disable interrupts */
    disable();                            /* Same as: asm cli; */

    /* Write color palette register index to read from */
    outp(PALETTE_READ, 0);

    /* Let's get our colors */
    for (i = 0; i < ScreenColors; i++) {

        /* Read color palette entry */
        color[i].color     = 0L;
        color[i].RGB.red   = inp(PALETTE_DATA);
        color[i].RGB.green = inp(PALETTE_DATA);
        color[i].RGB.blue  = inp(PALETTE_DATA);

    }

    /* Enable interrupts */
    enable();                             /* Same as: asm sti; */

    return color;                         /* Return our color array */
}
