#include "include\picture.hpp"

Picture::Picture(int w, int h, bool useVideoMem)
{
    if (useVideoMem) {
        width  = (w < 1 || w > ScreenWidth)  ? ScreenWidth : w;
        height = (w < 1 || h > ScreenHeight) ? ScreenHeight : h;
        image  = videoMem;
    } else if (w || h) {
        image = (unsigned char far *)calloc(w * h, sizeof(char));
        if (image == NULL) {
            width  = 0;
            height = 0;
            initOk = false;
        } else {
            width  = w;
            height = h;
            initOk = true;
        }
    }

    width = (width & 1) ? (width + 1) : width;
    displayMem = videoMem;
    strcpy(fileFormat, "PCX");
    subFormat  = 0;
    GetPaletteUsing(palette);
}

Picture::~Picture(void)
{
    if (image != videoMem && image != NULL)
        farfree(image);
}

bool Picture::LoadImageFromFile(const char *fileName)
{
    char *tempStr;
    ifstream fin(fileName, ios::in | ios::binary);
    if (!fin.good()) return false;

    tempStr = strchr(fileName, '.');
    if (tempStr != NULL) {
        tempStr = strupr(&tempStr[1]);
        if (!strcmp(tempStr, "PCX") || !strcmp(tempStr, "BMP"))
            strcpy(fileFormat, tempStr);
    }

    if (!strcmp(fileFormat, "PCX")) return ReadPCXFile(fin);
    return false;
}

bool Picture::SaveImageToFile(const char *fileName)
{
    char *tempStr;
    ofstream fout(fileName, ios::out | ios::binary | ios::trunc);
    if (!fout.good()) return false;

    tempStr = strchr(fileName, '.');
    if (tempStr != NULL) {
        tempStr = strupr(&tempStr[1]);
        if (!strcmp(tempStr, "PCX") || !strcmp(tempStr, "BMP"))
            strcpy(fileFormat, tempStr);
    }

    if (!strcmp(fileFormat, "PCX")) return WritePCXFile(fout);
    return false;
}

void Picture::DisplayImage(int xOffset, int yOffset)
{
    int startX, startY;
    int lenXcopy, lenYcopy, i;
    unsigned char far *dest;
    unsigned char far *src;

    if (image == NULL) return;
    if (xOffset < 0)
        startX = -xOffset;
    else
        startX = 0;

    if (startX < width) {
        if (xOffset + width > ScreenWidth)
            lenXcopy = ScreenWidth - (xOffset + startX);
        else
            lenXcopy = width - startX;
    } else
        lenXcopy = 0;

    if (lenXcopy == 0) return;

    if (yOffset < 0)
        startY = -yOffset;
    else
        startY = 0;

    if (startY < height) {
        if (yOffset + height > ScreenHeight)
            lenYcopy = ScreenHeight - (yOffset + startY);
        else
            lenYcopy = height - startY;
    } else
        lenYcopy = 0;

    if (lenYcopy == 0) return;

    src  = &image[startY * width + startX];
    dest = &displayMem[((yOffset + startY) * ScreenWidth)+(xOffset + startX)];
    for (i = 0; i < lenYcopy; i++) {
        _fmemcpy(dest, src, lenXcopy);
        src  += width;
        dest += ScreenWidth;
    }
}

void Picture::InitPCXHeader(void)
{
    memset(&header.PCX, 0, sizeof(header.PCX));

    header.PCX.manufacturer = 10;           // 10 - for ZSoft
    header.PCX.version      = 5;            // Version 5 of PC Paintbrush
    header.PCX.encoding     = 1;            // 1 - for PCX run length encoding
    header.PCX.bitsPerPixel = 8;            // 8 bits per pixel in Mode 13H
    header.PCX.xMax         = width  - 1;   // Maximum X-coordinate value
    header.PCX.yMax         = height - 1;   // Maximum Y-coordinate value
    header.PCX.horizRes     = ScreenWidth;  // Screen resolution width
    header.PCX.vertRes      = ScreenHeight; // Screen resolution height

    for (int i = 0; i < 16; i++) {          // Initialize the EGA palette
        header.PCX.egaPalette[i][0] = palette[i].RGB.red   * 255 / 63;
        header.PCX.egaPalette[i][1] = palette[i].RGB.green * 255 / 63;
        header.PCX.egaPalette[i][2] = palette[i].RGB.blue  * 255 / 63;
    }

    header.PCX.numOfPlanes  = 1;            // Only one plane in Mode 13H
    header.PCX.bytesPerLine = ((width & 1) ? (width + 1) : width);
    header.PCX.paletteType  = 1;            // Color palette
    header.PCX.hScreenSize  = ScreenWidth;  // Screen horizontal size
    header.PCX.vScreenSize  = ScreenHeight; // Screen vertical size

    strncpy(header.PCX.filler,              // ID String in filler space
            "Created by the Picture class by Gary Neal, Jr.", 54);
    header.PCX.filler[53]   = '\0';         // Null terminate ID string
}

bool Picture::WritePCXFile(ofstream &fout)
{
    int row, col, count;
    unsigned char data;

    InitPCXHeader();
    fout.write((char *)&header.PCX, sizeof(header.PCX));

    for (row = 0; row < height; row++) {
        count = 0;
        for (col = 0; col < width; col++) {
            if (!col) data = image[row * width + col];
            if (data == image[row * width + col]) {
                count++;
                if (count == 0x3F) {
                    fout.put(count | 0xC0);
                    fout.put(data);
                    count = 0;
                }
            } else {
                if (!count) {
                    count = 1;
                    data = image[row * width + col];
                } else if (count > 1) {
                    fout.put(count | 0xC0);
                    fout.put(data);
                    count = 1;
                    data = image[row * width + col];
                } else if ((data & 0xC0) == 0xC0) {
                    fout.put(count | 0xC0);
                    fout.put(data);
                    count = 1;
                    data = image[row * width + col];
                } else {
                    fout.put(data);
                    count = 1;
                    data = image[row * width + col];
                }
            }
        }

        if (count)
            if (count > 1) {
                fout.put(count | 0xC0);
                fout.put(data);
            } else if ((data & 0xC0) == 0xC0) {
                fout.put(count | 0xC0);
                fout.put(data);
            } else
                fout.put(data);
    }

    fout.put(0x0C);
    for (count = 0; count < ScreenColors; count++) {
        fout.put(palette[count].RGB.red   * 255 / 63);
        fout.put(palette[count].RGB.green * 255 / 63);
        fout.put(palette[count].RGB.blue  * 255 / 63);
    }
    return true;
}

bool Picture::ReadPCXFile(ifstream &fin)
{
    int row, col, count;
    unsigned char data;
    unsigned char far *tempPtr;

    fin.read((char *)&header.PCX, sizeof(header.PCX));

    if (header.PCX.manufacturer != 10 || header.PCX.bitsPerPixel != 8 ||
        header.PCX.version      != 5  || header.PCX.numOfPlanes  != 1)
        return false;

    width  = header.PCX.numOfPlanes * header.PCX.bytesPerLine;
    height = header.PCX.yMax - header.PCX.yMin + 1;
    if (image != videoMem) {
        if (image != NULL) farfree(image);
        image = (unsigned char far *)farcalloc(width * height, sizeof(char));
        if (image == NULL) return false;
    } else {
        if (width  > ScreenWidth)  width  = ScreenWidth;
        if (height > ScreenHeight) height = ScreenHeight;
    }

    for (row = 0; row < height; row++) {
        if (image == videoMem)
            tempPtr = &image[row * ScreenWidth];
        else
            tempPtr = &image[row * width];
        col = 0;
        while (col < width) {
            data = fin.get();
            if ((data & 0xC0) == 0xC0) {
                count = (data & 0x3F);
                data  = fin.get();
            } else
                count = 1;
            while (count && col < width) {
                tempPtr[col] = data;
                col++;
                count--;
            }
        }
    }

    fin.seekg(-769, ios::end);
    if (fin.get() != 12)
        GetPaletteUsing(palette);
    else {
        for (count = 0; count < ScreenColors; count++) {
            palette[count].RGB.red   = fin.get() >> 2;
            palette[count].RGB.green = fin.get() >> 2;
            palette[count].RGB.blue  = fin.get() >> 2;
        }
    }
    return true;
}
