/******************************************************************************
 *                             BASIC SPRITE CLASS                             *
 *                              by Gary Neal, Jr.                             *
 *               garyneal_71@yahoo.com       garyneal@geocities.com           *
 ******************************************************************************/

#include "include\sprite.hpp"

/******************************************************************************/

// Auto initialize shared variables
unsigned int       Sprite::spriteCount = 0;
unsigned char far *Sprite::displayMem  = videoMem;

/******************************************************************************/

Sprite::Sprite(int w, int h, unsigned int n, bool useBG, bool ownImages)
{
    X = Y = lastX = lastY = 0;           // Set the screen coordinates.

    width = height = 0;                  // Clear width and height.

    totalNumOfImages = imageIndex = 0;   // Clear the index and image count.
    images = NULL;                       // Set image array to nothing.
    imagesOwned = ownImages;             // Set ownership status

    background = NULL;                   // Set background to nothing.

    visible = false;                     // Set images to invisible.
    usesTransparentPixels = true;        // Set transparency status true.

    initOk = SetTotalNumOfImages(n);     // Create images array.
    initOk = SetDimensions(w, h, useBG); // Allocate memory for images.

    spriteCount++;                       // Increment sprite count
}

/******************************************************************************/

Sprite::Sprite(const Sprite &s)
{
    X     = s.X;                         // Set X coordinate.
    Y     = s.Y;                         // Set Y coordinate.
    lastX = s.lastX;                     // Set previous X coordinate.
    lastY = s.lastY;                     // Set previous Y coordinate.

    width  = s.width;                    // Set image width.
    height = s.height;                   // Set image height.

    totalNumOfImages = imageIndex = 0;   // Set index and image count.
    images = NULL;                       // Set image array to nothing.
    imagesOwned = false;                 // Set ownership to false.

    background = NULL;                   // Set background to nothing.

    visible = false;                     // Make image invisible.
    usesTransparentPixels = s.usesTransparentPixels;

    // Create images array and copy other sprite's array.
    if (initOk = SetTotalNumOfImages(s.totalNumOfImages)) {
        memcpy(images, s.images, totalNumOfImages*sizeof(unsigned char far *));
        imageIndex = s.imageIndex;
    }

    // Set dimensions and copy other sprite's background.
    if (s.background != NULL) {
        if (initOk = SetDimensions(width, height, true)) {
            _fmemcpy(background, s.background, width * height * sizeof(char));
            visible = s.visible;
        }
    }

    spriteCount++;                       // Increment sprite count.
}

/******************************************************************************/

Sprite::~Sprite(void)
{
    unsigned int i;

    // Destroy all images.
    for (i = 0; i < totalNumOfImages; i++)
        if (images[i] != NULL && imagesOwned)
            farfree(images[i]);

    // Destroy images array.
    if (images != NULL) free(images);

    // Destroy background buffer
    if (background != NULL) farfree(background);

    // Decrement sprite count.
    spriteCount--;
}

/******************************************************************************/

Sprite &Sprite::operator=(const Sprite &s)
{
    Sprite::~Sprite();                   // Destroy this sprite.
    Sprite::Sprite(s);                   // Initialize to passed sprite.
    return (*this);                      // Return reference.
}

/******************************************************************************/

void Sprite::Move(int x, int y, unsigned int i)
{
    bool wasVisible;
    wasVisible = visible;

    lastX = X;
    lastY = Y;

    Hide();

    X = x;
    Y = y;

    if (i < totalNumOfImages) imageIndex = i;

    if (wasVisible) Show();
}

/******************************************************************************/

bool Sprite::SetDimensions(int w, int h, bool useBackground)
{
    unsigned int i;
    bool success = true;

    Hide();

    if (w < 0) w = 0;
    if (w > ScreenWidth)  w = ScreenWidth;
    if (h < 0) h = 0;
    if (h > ScreenHeight) h = ScreenHeight;

    if (w && h) {
        if (imagesOwned) {
            for (i = 0; i < totalNumOfImages; i++) {
                if (images[i] != NULL) farfree(images[i]);
                images[i] = (unsigned char far *)farcalloc(w * h, sizeof(char));
                if (images[i] == NULL) success = false;
            }
        }

        if (background != NULL) farfree(background);
        if (useBackground) {
            background = (unsigned char far *)farcalloc(w * h, sizeof(char));
            if (background == NULL) success = false;
        }
    } else {
        for (i = 0; i < totalNumOfImages; i++) {
            if (images[i] != NULL && imagesOwned) farfree(images[i]);
            images[i] = NULL;
        }

        if (background != NULL) farfree(background);
        background = NULL;
    }

    if (success) {
        width  = w;
        height = h;
    }

    return success;
}

/******************************************************************************/

bool Sprite::SetTotalNumOfImages(unsigned int n)
{
    unsigned int i;
    unsigned char far **temp;

    Hide();
    if (n == totalNumOfImages) return true;

    if (!n) {
        for (i = 0; i < totalNumOfImages; i++) {
            if (images[i] != NULL && imagesOwned) farfree(images[i]);
            images[i] = NULL;
        }
        if (images != NULL) free(images);
        images = NULL;
        totalNumOfImages = 0;
        return true;
    }

    temp = (unsigned char far **)calloc(n, sizeof(unsigned char far *));
    if (temp == NULL) return false;

    if (n > totalNumOfImages) {
        for (i = 0; i < totalNumOfImages; i++) temp[i] = images[i];
        while (i < n) {
            if (width && height && imagesOwned)
                temp[i] = (unsigned char far *)
                          farcalloc(width * height, sizeof(unsigned char));
            else
                temp[i] = NULL;
            i++;
        }
    } else {
        for (i = 0; i < n; i++) temp[i] = images[i];
        while (i < totalNumOfImages) {
            if (images[i] != NULL && imagesOwned) farfree(images[i]);
            i++;
        }
    }

    if (images != NULL) free(images);
    images = temp;
    totalNumOfImages = n;
    return true;
}

/******************************************************************************/

void Sprite::DisplayImage(unsigned int i)
{
    bool wasVisible;
    wasVisible = visible;

    Hide();
    SetImageIndex(i);
    if (wasVisible) Show();
}

/******************************************************************************/

void Sprite::SetIndexedImage(unsigned char far *img, unsigned int i)
{
    if (i < totalNumOfImages) {
        if (imagesOwned)
            _fmemcpy(images[i], img, width * height * sizeof(char));
        else
            images[i] = img;
    }
}

/******************************************************************************/

void Sprite::Hide(void)
{
    int i;
    unsigned char far *src;
    unsigned char far *dest;

    if (!visible) return;

    if (background != NULL) {
        src  = background;
        dest = &displayMem[(Y << 8) + (Y << 6) + X];

        for (i = 0; i < height; i++) {
            _fmemcpy(dest, src, width);
            src  += width;
            dest += ScreenWidth;
        }
    }

    visible = false;
}

/******************************************************************************/

void Sprite::Show(void)
{
    int i, t;
    unsigned char far *src;
    unsigned char far *dest;

    if (visible) return;
    visible = true;

    if (images == NULL || images[imageIndex] == NULL) return;

    if (background != NULL) {
        src  = &displayMem[(Y << 8) + (Y << 6) + X];
        dest = background;

        for (i = 0; i < height; i++) {
            _fmemcpy(dest, src, width);
            src  += ScreenWidth;
            dest += width;
        }
    }

    src  = images[imageIndex];
    dest = &displayMem[(Y << 8) + (Y << 6) + X];

    if (usesTransparentPixels) {
        for (i = 0; i < height; i++) {
            for (t = 0; t < width; t++)
                if (src[t]) dest[t] = src[t];
            src  += width;
            dest += ScreenWidth;
        }
    } else {
        for (i = 0; i < height; i++) {
            _fmemcpy(dest, src, width);
            src  += width;
            dest += ScreenWidth;
        }
    }
}

/******************************************************************************/
